// %flair:license{
// This file is part of the Flair framework distributed under the
// CECILL-C License, Version 1.0.
// %flair:license}
//  created:    2014/02/07
//  filename:   SimulatedAhrs.cpp
//
//  author:     Guillaume Sanahuja
//              Copyright Heudiasyc UMR UTC/CNRS 7253
//
//  version:    $Id: $
//
//  purpose:    Class for a simulation ahrs
//
//
/*********************************************************************/
#ifdef CORE2_64

#include "SimulatedAhrs.h"
#include "SimulatedImu.h"
#include <AhrsData.h>

using std::string;
using namespace flair::core;
using namespace flair::sensor;

namespace flair {
namespace filter {

SimulatedAhrs::SimulatedAhrs(string name, uint32_t modelId,uint32_t deviceId,
                   uint8_t priority)
    : Ahrs(new SimulatedImu("imu", modelId,deviceId, priority), name) {
  SetIsReady(true);
}

SimulatedAhrs::~SimulatedAhrs() {}

// datas from SimulatedImu are AhrsData!
void SimulatedAhrs::UpdateFrom(const io_data *data) {
  AhrsData *input = (AhrsData *)data;
  AhrsData *output;
  GetDatas(&output);

  Quaternion quaternion;
  Vector3Df filteredAngRates;
  input->GetQuaternionAndAngularRates(quaternion, filteredAngRates);
  output->SetQuaternionAndAngularRates(quaternion, filteredAngRates);
  output->SetDataTime(input->DataTime());

  ProcessUpdate(output);
}

} // end namespace filter
} // end namespace flair

#endif