// %flair:license{
// This file is part of the Flair framework distributed under the
// CECILL-C License, Version 1.0.
// %flair:license}
/*!
 * \file SimulatedAhrs.h
 * \brief Class for a simulation Ahrs
 * \author Guillaume Sanahuja, Copyright Heudiasyc UMR UTC/CNRS 7253
 * \date 2014/02/07
 * \version 4.0
 */

#ifndef SIMUAHRS_H
#define SIMUAHRS_H

#include <Ahrs.h>
#include <stdint.h>

namespace flair {
namespace filter {
/*! \class SimulatedAhrs
*
* \brief Class for a simulation Ahrs
*
* This class constructs a SimulatedImu as Imu of this Ahrs.
*/
class SimulatedAhrs : public filter::Ahrs {
public:
  /*!
  * \brief Constructor
  *
  * Construct a simulation Ahrs. It will be child of the FrameworkManager.
  *
  * \param name name
  * \param modelId Model id
  * \param deviceId Ahrs id of the Model
  * \param priority priority of the SimulatedImu Thread
  */
  SimulatedAhrs(std::string name,
           uint32_t modelId,uint32_t deviceId, uint8_t priority);

  /*!
  * \brief Destructor
  *
  */
  ~SimulatedAhrs();

private:
  /*!
  * \brief Update using provided datas
  *
  * Reimplemented from IODevice.
  *
  * \param data data from the parent to process
  */
  void UpdateFrom(const core::io_data *data);
};
} // end namespace filter
} // end namespace flair
#endif // SIMUAHRS_H
