// %flair:license{
// This file is part of the Flair framework distributed under the
// CECILL-C License, Version 1.0.
// %flair:license}
/*!
 * \file EulerDerivative.h
 * \brief Classe permettant le calcul d'une derivee d'Euler
 * \author Guillaume Sanahuja, Copyright Heudiasyc UMR UTC/CNRS 7253
 * \date 2011/05/01
 * \version 4.0
 */

#ifndef EULERDERIVATIVE_IMPL_H
#define EULERDERIVATIVE_IMPL_H

#include <IODevice.h>

namespace flair {
namespace core {
class Matrix;
}
namespace gui {
class LayoutPosition;
class DoubleSpinBox;
}
namespace filter {
class EulerDerivative;
}
}

/*! \class EulerDerivative
* \brief Classe permettant le calcul d'une derivee d'Euler
*/

class EulerDerivative_impl {
public:
  EulerDerivative_impl(flair::filter::EulerDerivative *self,
                       const flair::gui::LayoutPosition *position,
                       std::string name,
                       const flair::core::Matrix *init_value = NULL);
  ~EulerDerivative_impl();
  void UpdateFrom(const flair::core::io_data *data);
  flair::core::Matrix *output;

private:
  flair::gui::DoubleSpinBox *T,*sat;
  bool first_update;
  flair::core::Matrix *prev_input,*prev_output;
  flair::filter::EulerDerivative *self;
};

#endif // EULERDERIVATIVE_IMPL_H
