//  created:    2014/07/17
//  filename:   CvtColor.cpp
//
//  author:     Guillaume Sanahuja
//              Copyright Heudiasyc UMR UTC/CNRS 7253
//
//  version:    $Id: $
//
//  purpose:    Image color conversion
//
//
/*********************************************************************/

#include "CvtColor.h"
#include <Image.h>
#include <typeinfo>

using std::string;
using namespace flair::core;

namespace flair { namespace filter {

CvtColor::CvtColor(const core::IODevice* parent,std::string name,Conversion_t conversion) : IODevice(parent,name),output(0) {
    this->conversion=conversion;

    switch(conversion) {
    case Conversion_t::ToGray:
        try{
            Image::Type const &imageType=dynamic_cast<Image::Type const &>(parent->GetOutputDataType());
            output=new Image(this,imageType.GetWidth(),imageType.GetHeight(),Image::Type::Format::Gray,"conversion",true,2);

        } catch(std::bad_cast& bc) {
            Err("io type mismatch\n");
        }
        break;
    default:
        Err("conversion not supported\n");
    }
    
    SetIsReady(true);
}

CvtColor::~CvtColor(void) {}

Image* CvtColor::Output(void) {
    return output;

}

DataType const &CvtColor::GetOutputDataType() const {
    if(output!=NULL) {
        return output->GetDataType();
    } else {
        return dummyType;
    }
}

void CvtColor::UpdateFrom(const io_data *data) {
    Image *img=(Image*)data;

    data->GetMutex();
    output->GetMutex();

    switch(conversion) {
    case Conversion_t::ToGray:
        switch(img->GetDataType().GetFormat()) {
        case Image::Type::Format::YUYV:
            //dspCvtColor(img,output->img,DSP_YUYV2GRAY);
            break;
        case Image::Type::Format::UYVY:
            //dspCvtColor(img,output->img,DSP_UYVY2GRAY);
            break;
        case Image::Type::Format::BGR:
            //dspCvtColor(img,output->img,DSP_BGR2GRAY);
            break;
        default:
            Err("input format not supported\n");
        }
        break;
    default:
        Err("conversion not supported\n");
    }

    output->ReleaseMutex();
    data->ReleaseMutex();

    output->SetDataTime(data->DataTime());
    ProcessUpdate(output);
}

} // end namespace filter
} // end namespace flair
