// %flair:license{
// This file is part of the Flair framework distributed under the
// CECILL-C License, Version 1.0.
// %flair:license}
#include "ConnectionLayout.h"
#include "UdtSocket.h"
#include "DataRemote.h"
#include <qendian.h>
#include "communication.h"

ConnectionLayout::ConnectionLayout(UdtSocket *socket, QString name)
    : Layout(NULL, name, "root") {
  this->socket = socket;
  this->name = name;
}

ConnectionLayout::~ConnectionLayout() {
}

void ConnectionLayout::udtSocketDestroyed(QObject *obj){
  socket=NULL;
}

void ConnectionLayout::receive(char *buf, int size) {
   //fprintf(stderr,"trame %x\n",buf[0]);
  // for(int i=0; i<size;i++) fprintf(stderr,"%x ",buf[i]);
  // fprintf(stderr,"\n");
  switch ((unsigned char)buf[0]) {
  case XML_HEADER: {
    QString xml;
    QDomDocument doc;
    xml = QString((char *)buf);
    xml.resize(size);

    // fprintf(stderr,"recu %i\n%s\n",size,xml.toLocal8Bit().constData());
    if (!doc.setContent(xml)) {
      fprintf(stderr,"prob setContent fichier\n");
    }

    QDomElement dom=doc.firstChildElement("root").firstChildElement();
    ParseXml(&dom);
    break;
  }
  case DATA_BIG_ENDIAN: {
    // for(int i=0;i<size;i++) fprintf(stderr,"%x ",buf[i]);
    // fprintf(stderr,"\n");
    uint16_t period;
    memcpy(&period, &buf[1], sizeof(uint16_t));
    period = qFromBigEndian(period);
    drawDatas(&buf[3], size - 3, period, true);
    break;
  }
  case DATA_LITTLE_ENDIAN: {
    // for(int i=0;i<size;i++) fprintf(stderr,"%x ",buf[i]);
    // fprintf(stderr,"\n");
    uint16_t period;
    memcpy(&period, &buf[1], sizeof(uint16_t));
    // fprintf(stderr,"recu %i period %i\n",size,period);
    drawDatas(&buf[3], size - 3, period);
    break;
  }
  case CLOSING_CONNECTION: {
    deleteLater();
    break;
  }
  default:
    fprintf(stderr,"trame non supportée %x\n", buf[0]);
  }
}

void ConnectionLayout::XmlToSend(QDomDocument doc) {
   //fprintf(stderr,"xml to send\n%s\n",doc.toString().toLocal8Bit().constData());
  if(!socket) return;
  
  // xml to send a mettre dans le manager
  if(doc.toString().toLocal8Bit().length()!=0) {
    socket->write(doc.toString().toLocal8Bit().constData(),doc.toString().toLocal8Bit().length());
  }
  /*
  QMetaObject::invokeMethod(
      socket, "write", Qt::BlockingQueuedConnection,
      Q_ARG(const char *, doc.toString().toLocal8Bit().constData()),
      Q_ARG(qint64, doc.toString().toLocal8Bit().length()));*/
}

void ConnectionLayout::LoadXml(QDomDocument *to_parse) {
  QDomElement tmp = to_parse->firstChildElement("root");
  while (tmp.attribute("name") != name && !tmp.isNull())
    tmp = to_parse->nextSiblingElement("root");

  if (!tmp.isNull()) {
    XmlWidget::LoadXml(&tmp);
  } else {
    fprintf(stderr,"%s not found in xml file \n", name.toLocal8Bit().constData());
  }
}

void ConnectionLayout::removeDataRemote(DataRemote *data) {
  dataremotes.removeOne(data);
}

void ConnectionLayout::addDataRemote(DataRemote *data) {
  dataremotes.append(data);
}

QString ConnectionLayout::getName() { return name; }

void ConnectionLayout::drawDatas(char *buf, int buf_size, uint16_t period,
                                 bool big_endian) {
  for (int i = 0; i < dataremotes.count(); i++) {
    dataremotes.at(i)->BufEvent(&buf, &buf_size, period, big_endian);
  }
}

QString ConnectionLayout::getDocRootName(char* buf, int size) {
  QString xml;
  QDomDocument doc;
  xml = QString((char *)buf);
  xml.resize(size);

  if (!doc.setContent(xml)) {
    fprintf(stderr,"prob setContent fichier\n");
  }
  
  return doc.firstChildElement("root").attribute("name");
}