// %flair:license{
// This file is part of the Flair framework distributed under the
// CECILL-C License, Version 1.0.
// %flair:license}
#include "DataRemote.h"
#include <QMenu>
#include <QInputDialog>
#include <QtXml>
#include "Manager.h"
#include "ConnectionLayout.h"

DataRemote::DataRemote(QString name, QString type, XmlWidget *parent,
                       bool enabled, int period)
    : XmlWidget(name, type, parent) {
  auto_refresh = enabled;
  is_logging = true;
  receivesize = 0;
  refresh_rate = (double)period / 1000.;

  connectionLayout()->addDataRemote(this);

  // pour ne pas faire de doublons qd on change la periode
  SetIsExpandable(true);
}

DataRemote::~DataRemote() { connectionLayout()->removeDataRemote(this); }

void DataRemote::appendmenu(QMenu *menu) {
  menu->addSeparator();

  autoRefresh = menu->addAction("auto refresh");
  autoRefresh->setCheckable(true);
  autoRefresh->setChecked(auto_refresh);

  setRefreshRate = menu->addAction(QString("set refresh rate (%1ms)")
                          .arg((uint16_t)(qRound(refresh_rate * 1000))));
  setRefreshRate->setEnabled(auto_refresh);
/*  menu->addSeparator();

  log = menu->addAction("log");
  log->setCheckable(true);
  log->setChecked(is_logging);
  */
}

QAction *DataRemote::execmenu(QWidget *parent, QMenu *menu, QPoint point) {
  QAction *action;

  action = menu->exec(point);

  // auto_refresh=b->isChecked();

  if (action == autoRefresh) {
    SendPeriod(RefreshRate_ms(), autoRefresh->isChecked());
  }

  if (action == setRefreshRate) {
    bool ok;
  
    uint16_t time = QInputDialog::getInt(
        parent, "Set refresh rate ", "Value (ms):",
        (uint16_t)(qRound(refresh_rate * 1000)), 1, 65535, 10, &ok);
    if (ok == true && time != qRound(refresh_rate * 1000)) {
      // refresh_rate=time/1000.;
      SendPeriod(time, autoRefresh->isChecked());
    }
  }
  
  return action;
}

uint16_t DataRemote::RefreshRate_ms(void) {
  return (uint16_t)(qRound(refresh_rate * 1000.));
}

bool DataRemote::IsEnabled(void) { return auto_refresh; }

void DataRemote::SendPeriod(int period, bool auto_refresh) {
  RemoveAllAttributes();

  SetAttribute("period", period);
  SetAttribute("enabled", auto_refresh);
  connectionLayout()->XmlToSend(XmlDoc());
  RemoveAttribute("period");
  RemoveAttribute("enabled");
}

int DataRemote::ReceiveSize(void) { return receivesize; }

void DataRemote::XmlSetup(QDomElement *dom) {
  refresh_rate = dom->attribute("period").toInt() / 1000.;
  if (dom->attribute("enabled") == "1")
    auto_refresh = true;
  else
    auto_refresh = false;
}
